package at.bestsolution.rsd.rt.opentracing.jaeger;

import java.util.Map;

import org.eclipse.microprofile.config.ConfigProvider;
import org.osgi.service.component.annotations.Component;

import at.bestsolution.rsd.rt.opentracing.TraceProvider;
import io.jaegertracing.Configuration;
import io.jaegertracing.Configuration.ReporterConfiguration;
import io.jaegertracing.Configuration.SamplerConfiguration;
import io.jaegertracing.Configuration.SenderConfiguration;
import io.jaegertracing.internal.samplers.ConstSampler;
import io.opentracing.Tracer;

@Component
public class JaegerTracingProvider implements TraceProvider {
	private volatile Tracer tracer;
	
	@Override
	public String getId() {
		return "jaeger";
	}

	private String getComponentName() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.component.name", String.class)
				.orElse("RSD Trace");
	}

	private String getHost() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.jaeger.host", String.class)
				.orElse("localhost");
	}

	private Integer getPort() {
		return ConfigProvider.getConfig().getOptionalValue("rsd.opentracing.jaeger.port", Integer.class).orElse(5775);
	}

	@Override
	public Tracer getTracer() {
		// XXX Detect configuration changes
		Tracer localTracer = tracer;
		if (localTracer == null) {
			synchronized (this) {
				localTracer = tracer;
				if (localTracer == null) {
					SamplerConfiguration samplerConfig = new SamplerConfiguration().withType(ConstSampler.TYPE)
							.withParam(1);
					SenderConfiguration senderConfig = new SenderConfiguration().withAgentHost(getHost())
							.withAgentPort(getPort());
					ReporterConfiguration reporterConfig = new ReporterConfiguration().withLogSpans(true)
							.withFlushInterval(1000).withMaxQueueSize(10000).withSender(senderConfig);
					tracer = localTracer = new Configuration(getComponentName()).withSampler(samplerConfig)
							.withReporter(reporterConfig).getTracer();
				}
			}
		}

		return localTracer;
	}
	
	@Override
	public Map<String, String> spanContextMap(String jsonString){
		Map<String, String> map = TraceProvider.super.spanContextMap(jsonString);
		//traceparent == zipkin key, so needs to be changed to jaeger
		if(map.containsKey("traceparent")) {
			String jsonZipkinString = map.remove("traceparent");
			String[] parts = jsonZipkinString.split("-");
			map.put("uber-trace-id", parts[1] + ":" + parts[2] + ":" + Integer.parseInt(parts[0], 10) + ":"
					+ Integer.parseInt(parts[3], 10));
		}
		return map;
	}

}
